<?php

/**
 * @file
 * Admin only functionality for the media_kaltura module.
 */

/**
 * Menu form callback; Display the media_kaltura admin form.
 */
function media_kaltura_admin_form($form, $form_state) {
  $mimetypes = media_kaltura_get_mimetypes();
  $settings = variable_get('media_kaltura', array(
    'general' => array(
      'upload' => FALSE,
      'edit' => FALSE,
      'delete' => FALSE,
      'default' => 0,
      'mimetypes' => implode("\n", $mimetypes),
    ),
  ));

  // Check that all libraries exist.
  $required_libraries = array('KalturaClient');
  foreach ($required_libraries as $name) {
    $library = libraries_detect($name);
    if (!$library['installed']) {
      drupal_set_message($library['error message'], 'error');
    }
  }

  // Load existing servers and add to table options array.
  $rows = array();
  $options = array();
  $servers = entity_load('media_kaltura_server');
  foreach ($servers as $server) {
    $rows[$server->id] = array(
      'domain' => $server->domain,
      'force_ssl' => $server->force_ssl ? 'Yes' : 'No',
      'api' => $server->api ? 'Enabled' : 'Disabled',
      'partner_id' => $server->partner_id,
      'subpartner_id' => $server->subpartner_id,
      'user_id' => $server->user_id,
      'secret' => $server->secret,
      'uiconf_id' => $server->uiconf_id,
      'operations' => l(t('edit'), 'admin/config/media/media_kaltura/server/' . $server->id),
    );
    $options[$server->id] = $server->domain;
  }

  $form['media_kaltura'] = array(
    '#type' => 'container',
    '#tree' => TRUE,
  );
  $form['media_kaltura']['servers'] = array(
    '#type' => 'fieldset',
    '#title' => t('Kaltura Servers'),
  );
  $form['media_kaltura']['servers']['list'] = array(
    '#theme' => 'table',
    '#header' => array(
      'domain' => t('Domain'),
      'force_ssl' => t('Force SSL'),
      'api' => t('Kaltura Client'),
      'partner_id' => t('Partner ID'),
      'subpartner_id' => t('Sub Partner ID'),
      'user_id' => t('User ID'),
      'secret' => t('User Secret'),
      'uiconf_id' => t('UI Conf ID'),
      'operations' => t('Operations'),
    ),
    '#rows' => $rows,
    '#empty' => t('No servers added yet.'),
  );
  $form['media_kaltura']['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General Settings'),
  );
  $form['media_kaltura']['general']['upload'] = array(
    '#type' => 'checkbox',
    '#title' => t('Automatically upload files to Kaltura server if they match an allowed mimetype?'),
    '#description' => t('All local files will be automatically uploaded to the Kaltura server if they match one of the allowed mimetypes below.<br /><br /><strong>Note: This will override all file save operations. Be sure this is the expected outcome before enabling this option!</strong>'),
    '#default_value' => $settings['general']['upload'],
  );
  $form['media_kaltura']['general']['default'] = array(
    '#type' => 'select',
    '#title' => t('Default server to upload files to?'),
    '#description' => t('Files will be automatically uploaded to the selected Kaltura server.'),
    '#default_value' => $settings['general']['default'],
    '#options' => $options,
    '#states' => array(
      'disabled' => array(
        ':input[name="media_kaltura[general][upload]"]' => array('checked' => FALSE),
      ),
    ),
  );
  $form['media_kaltura']['general']['mimetypes'] = array(
    '#type' => 'textarea',
    '#title' => t('Allowed mimetypes'),
    '#description' => t('Each mimetype should be separated by a new line. If you add mimetypes make sure they are supported by your Kaltura server.'),
    '#default_value' => $settings['general']['mimetypes'],
  );
  $form['media_kaltura']['general']['edit'] = array(
    '#type' => 'checkbox',
    '#title' => t('Include Kaltura media entry field on the file edit form?'),
    '#description' => t('This will add description and categories fields to the file edit form, and update the Kaltura media entry when the form is submitted.'),
    '#default_value' => $settings['general']['edit'],
  );
  $form['media_kaltura']['general']['delete'] = array(
    '#type' => 'checkbox',
    '#title' => t('Delete the corresponding Kaltura media entry when deleting the Drupal managed file?'),
    '#description' => t('This will delete the media entry and file on the Kaltura server when the managed file is deleted on your Drupal site.'),
    '#default_value' => $settings['general']['delete'],
  );

  $form = system_settings_form($form);

  $form['actions']['reset'] = array(
    '#type' => 'submit',
    '#value' => 'Reset mimetypes to default',
    '#submit' => array('media_kaltura_admin_form_reset'),
  );

  return $form;
}

/**
 * Validate handler for media_kaltura_admin_form().
 */
function media_kaltura_admin_form_validate($form, &$form_state) {
  if ($form_state['values']['media_kaltura']['general']['upload']) {

    $server = media_kaltura_server_load($form_state['values']['media_kaltura']['general']['default']);
    if (!$server) {
      form_set_error('media_kaltura][general][default', t('Unable to load server. Please check that your server information is correct.'));
    }

    // Start a new session with the Kaltura server.
    $kaltura = media_kaltura_start_session($server);
    if (!$kaltura) {
      form_set_error('media_kaltura][general][default', t('Unable to connect to server. Please check that your server information is correct.'));
    }
  }
}

/**
 * Custom submit handler for media_kaltura_admin_form().
 */
function media_kaltura_admin_form_reset($form, &$form_state) {
  $mimetypes = media_kaltura_get_mimetypes();
  $settings = variable_get('media_kaltura');
  $settings['general']['mimetypes'] = implode("\n", $mimetypes);
  variable_set('media_kaltura', $settings);
}

/**
 * Menu form callback; Add/Edit a kaltura server.
 */
function media_kaltura_server_edit_form($form, &$form_state, $domain = '') {
  $server = $domain ? media_kaltura_server_load($domain) : (object) array(
    'id' => 'new',
    'domain' => '',
    'force_ssl' => FALSE,
    'api' => FALSE,
    'partner_id' => '',
    'subpartner_id' => '',
    'user_id' => '',
    'secret' => '',
    'uiconf_id' => '',
  );

  $form['server'] = array(
    '#type' => 'container',
  );
  $form['server']['id'] = array(
    '#type' => 'hidden',
    '#value' => $server->id,
  );
  $form['server']['domain'] = array(
    '#type' => 'textfield',
    '#title' => t('Domain'),
    '#required' => TRUE,
    '#description' => t('The domain of the kaltura server. For example, use cdnapi.kaltura.com for the commercial version of Kaltura.'),
    '#default_value' => $server->domain,
    '#attributes' => array(
      'placeholder' => array(
        'cdnapi.kaltura.com',
      ),
    ),
  );
  $form['server']['force_ssl'] = array(
    '#type' => 'checkbox',
    '#title' => t('Force SSL for this server?'),
    '#default_value' => $server->force_ssl,
  );
  $form['server']['api'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use Kaltura Client for this server?'),
    '#description' => t('This will enable closer integration between the drupal managed file system and the Kaltura server. When a kaltura managed file is edited or deleted, the corresponding media entry on the Kaltura server will also be edited or deleted.'),
    '#default_value' => $server->api,
  );
  $form['server']['api_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Kaltura Client API Settings'),
    '#states' => array(
      'disabled' => array(
        ':input[name="api"]' => array('checked' => FALSE),
      ),
    ),
  );
  $form['server']['api_settings']['partner_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Partner ID'),
    '#description' => t('The partner ID of the account used to authenticate sessions with the Kaltura server.'),
    '#default_value' => $server->partner_id,
  );
  $form['server']['api_settings']['subpartner_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Sub Partner ID'),
    '#description' => t('The sub-partner ID of the account used to authenticate sessions with the Kaltura server.'),
    '#default_value' => $server->subpartner_id,
  );
  $form['server']['api_settings']['user_id'] = array(
    '#type' => 'textfield',
    '#title' => t('User ID'),
    '#description' => t('The ID of the user account used to authenticate sessions with the Kaltura server.'),
    '#default_value' => $server->user_id,
  );
  $form['server']['api_settings']['secret'] = array(
    '#type' => 'textfield',
    '#title' => t('User Secret Key'),
    '#description' => t('The secret key of the user account used to authenticate sessions with the Kaltura server.'),
    '#default_value' => $server->secret,
  );
  $form['server']['api_settings']['uiconf_id'] = array(
    '#type' => 'textfield',
    '#title' => t('UI Conf ID'),
    '#description' => t('The id of the player used to play videos from this account.'),
    '#default_value' => $server->uiconf_id,
  );
  $form['actions'] = array(
    '#type' => 'actions',
    'submit' => array(
      '#type' => 'submit',
      '#value' => t('Save Server'),
    ),
  );
  if (!empty($server->domain)) {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#submit' => array('media_kaltura_server_delete_form_submit'),
    );
  }

  return $form;
}

/**
 * Menu form callback; Add/Edit a kaltura server.
 */
function media_kaltura_server_edit_form_validate($form, &$form_state) {
  $server = (object) $form_state['values'];

  // Attempt to start session with server information if API is enabled.
  if ($server->api) {
    if (!media_kaltura_start_session($server)) {
      form_set_error('server', t('Unable to connect to server. Please check that your server information is correct.'));
    }
  }
}

/**
 * Menu form callback; Add/Edit a kaltura server.
 */
function media_kaltura_server_edit_form_submit($form, &$form_state) {
  $server = (object) $form_state['values'];

  // Save new server.
  if ($server->id === 'new') {
    drupal_write_record('media_kaltura_server', $server);
    drupal_set_message(t('New server successfully added.'));
  }

  // Update server.
  elseif (is_numeric($server->id)) {
    drupal_write_record('media_kaltura_server', $server, 'id');
    drupal_set_message(t('Server successfully updated.'));
  }

  $form_state['redirect'] = 'admin/config/media/media_kaltura';
}

/**
 * Menu form callback; Delete a kaltura server.
 */
function media_kaltura_server_delete_form_submit($form, &$form_state) {
  $server = (object) $form_state['values']['server'];

  if (is_numeric($server->id)) {
    if ($count = db_delete('media_kaltura_server')->condition('id', $server->id)->execute()) {
      drupal_set_message(t('Server successfully deleted.'));
    }
  }

  $form_state['redirect'] = 'admin/config/media/media_kaltura';
}
